@@ -74,6 +74,8 @@ static gboolean gst_aiff_parse_sink_activate (GstPad * sinkpad,
     GstObject * parent);
 static gboolean gst_aiff_parse_sink_activate_mode (GstPad * sinkpad,
     GstObject * parent, GstPadMode mode, gboolean active);
+static gboolean gst_aiff_parse_sink_event (GstPad * pad, GstObject * parent,
+    GstEvent * buf);
 static gboolean gst_aiff_parse_send_event (GstElement * element,
     GstEvent * event);
 static GstStateChangeReturn gst_aiff_parse_change_state (GstElement * element,
@@ -191,6 +193,8 @@ gst_aiff_parse_init (GstAiffParse * aiffparse)
       GST_DEBUG_FUNCPTR (gst_aiff_parse_sink_activate));
   gst_pad_set_activatemode_function (aiffparse->sinkpad,
       GST_DEBUG_FUNCPTR (gst_aiff_parse_sink_activate_mode));
+  gst_pad_set_event_function (aiffparse->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_aiff_parse_sink_event));
   gst_pad_set_chain_function (aiffparse->sinkpad,
       GST_DEBUG_FUNCPTR (gst_aiff_parse_chain));
   gst_element_add_pad (GST_ELEMENT_CAST (aiffparse), aiffparse->sinkpad);
@@ -438,11 +442,11 @@ gst_aiff_parse_perform_seek (GstAiffParse * aiff, GstEvent * event)
             aiff->segment.format, aiff->segment.position));
   }
 
-  /* now create the newsegment */
-  GST_DEBUG_OBJECT (aiff, "Creating newsegment from %" G_GINT64_FORMAT
+  /* now create the segment */
+  GST_DEBUG_OBJECT (aiff, "Creating segment from %" G_GINT64_FORMAT
       " to %" G_GINT64_FORMAT, aiff->segment.position, stop);
 
-  /* store the newsegment event so it can be sent from the streaming thread. */
+  /* store the segment event so it can be sent from the streaming thread. */
   if (aiff->start_segment)
     gst_event_unref (aiff->start_segment);
   aiff->start_segment = gst_event_new_segment (&aiff->segment);
@@ -575,8 +579,7 @@ gst_aiff_parse_calculate_duration (GstAiffParse * aiff)
 }
 
 static void
-gst_aiff_parse_ignore_chunk (GstAiffParse * aiff, GstBuffer * buf, guint32 tag,
-    guint32 size)
+gst_aiff_parse_ignore_chunk (GstAiffParse * aiff, guint32 tag, guint32 size)
 {
   guint flush;
 
@@ -590,8 +593,6 @@ gst_aiff_parse_ignore_chunk (GstAiffParse * aiff, GstBuffer * buf, guint32 tag,
   aiff->offset += flush;
   if (aiff->streaming) {
     gst_adapter_flush (aiff->adapter, flush);
-  } else {
-    gst_buffer_unref (buf);
   }
 }
 
@@ -708,7 +709,7 @@ gst_aiff_parse_read_chunk (GstAiffParse * aiff, guint64 * offset, guint32 * tag,
 {
   guint size;
   GstFlowReturn res;
-  GstBuffer *buf;
+  GstBuffer *buf = NULL;
   GstMapInfo info;
 
   if ((res =
@@ -720,6 +721,7 @@ gst_aiff_parse_read_chunk (GstAiffParse * aiff, guint64 * offset, guint32 * tag,
   size = GST_READ_UINT32_BE (info.data + 4);
   gst_buffer_unmap (buf, &info);
   gst_buffer_unref (buf);
+  buf = NULL;
 
   if ((res =
           gst_pad_pull_range (aiff->sinkpad, (*offset) + 8, size,
@@ -784,6 +786,7 @@ gst_aiff_parse_create_caps (GstAiffParse * aiff)
     caps = gst_caps_new_simple ("audio/x-raw",
         "format", G_TYPE_STRING, format,
         "channels", G_TYPE_INT, aiff->channels,
+        "layout", G_TYPE_STRING, "interleaved",
         "rate", G_TYPE_INT, aiff->rate, NULL);
   }
 
@@ -796,7 +799,7 @@ static GstFlowReturn
 gst_aiff_parse_stream_headers (GstAiffParse * aiff)
 {
   GstFlowReturn res;
-  GstBuffer *buf;
+  GstBuffer *buf = NULL;
   guint32 tag, size;
   gboolean gotdata = FALSE;
   gboolean done = FALSE;
@@ -823,6 +826,8 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
       tag = GST_READ_UINT32_LE (info.data);
       size = GST_READ_UINT32_BE (info.data + 4);
       gst_buffer_unmap (buf, &info);
+      gst_buffer_unref (buf);
+      buf = NULL;
     }
 
     GST_INFO_OBJECT (aiff,
@@ -834,6 +839,8 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
     switch (tag) {
       case GST_MAKE_FOURCC ('C', 'O', 'M', 'M'):{
         GstCaps *caps;
+        GstEvent *event;
+        gchar *stream_id;
 
         if (aiff->streaming) {
           if (!gst_aiff_parse_peek_chunk (aiff, &tag, &size))
@@ -858,6 +865,14 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
         if (aiff->rate == 0)
           goto no_rate;
 
+        stream_id =
+            gst_pad_create_stream_id (aiff->srcpad, GST_ELEMENT_CAST (aiff),
+            NULL);
+        event = gst_event_new_stream_start (stream_id);
+        gst_event_set_group_id (event, gst_util_group_id_next ());
+        gst_pad_push_event (aiff->srcpad, event);
+        g_free (stream_id);
+
         GST_DEBUG_OBJECT (aiff, "creating the caps");
 
         caps = gst_aiff_parse_create_caps (aiff);
@@ -896,7 +911,6 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
           GstBuffer *ssndbuf = NULL;
           GstMapInfo info;
 
-          gst_buffer_unref (buf);
           if ((res =
                   gst_pad_pull_range (aiff->sinkpad, aiff->offset, 16,
                       &ssndbuf)) != GST_FLOW_OK)
@@ -913,7 +927,7 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
 
         /* 8 byte chunk header, 8 byte SSND header */
         aiff->offset += 16;
-        datasize = size - 16;
+        datasize = size - 8;
 
         aiff->datastart = aiff->offset + aiff->ssnd_offset;
         /* file might be truncated */
@@ -968,9 +982,11 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
         break;
       }
       default:
-        gst_aiff_parse_ignore_chunk (aiff, buf, tag, size);
+        gst_aiff_parse_ignore_chunk (aiff, tag, size);
     }
 
+    buf = NULL;
+
     if (upstream_size && (aiff->offset >= upstream_size)) {
       /* Now we have gone through the whole file */
       done = TRUE;
@@ -1003,7 +1019,7 @@ gst_aiff_parse_stream_headers (GstAiffParse * aiff)
 
   /* now we have all the info to perform a pending seek if any, if no
    * event, this will still do the right thing and it will also send
-   * the right newsegment event downstream. */
+   * the right segment event downstream. */
   gst_aiff_parse_perform_seek (aiff, aiff->seek_event);
   /* remove pending event */
   event_p = &aiff->seek_event;
@@ -1494,20 +1510,19 @@ done:
 static gboolean
 gst_aiff_parse_pad_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
-  gboolean res = TRUE;
+  gboolean res = FALSE;
   GstAiffParse *aiff = GST_AIFF_PARSE (parent);
 
-  /* only if we know */
-  if (aiff->state != AIFF_PARSE_DATA) {
-    return FALSE;
-  }
-
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_DURATION:
     {
       gint64 duration = 0;
       GstFormat format;
 
+      /* only if we know */
+      if (aiff->state != AIFF_PARSE_DATA)
+        break;
+
       gst_query_parse_duration (query, &format, NULL);
 
       switch (format) {
@@ -1530,6 +1545,10 @@ gst_aiff_parse_pad_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gint64 srcvalue, dstvalue;
       GstFormat srcformat, dstformat;
 
+      /* only if we know */
+      if (aiff->state != AIFF_PARSE_DATA)
+        break;
+
       gst_query_parse_convert (query, &srcformat, &srcvalue,
           &dstformat, &dstvalue);
       res = gst_aiff_parse_pad_convert (pad, srcformat, srcvalue,
@@ -1541,6 +1560,10 @@ gst_aiff_parse_pad_query (GstPad * pad, GstObject * parent, GstQuery * query)
     case GST_QUERY_SEEKING:{
       GstFormat fmt;
 
+      /* only if we know */
+      if (aiff->state != AIFF_PARSE_DATA)
+        break;
+
       gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
       if (fmt == GST_FORMAT_TIME) {
         gboolean seekable = TRUE;
@@ -1622,13 +1645,17 @@ gst_aiff_parse_sink_activate_mode (GstPad * sinkpad, GstObject * parent,
   gboolean res;
   GstAiffParse *aiff = GST_AIFF_PARSE (parent);
 
-  if (aiff->adapter)
+  if (aiff->adapter) {
     g_object_unref (aiff->adapter);
+    aiff->adapter = NULL;
+  }
 
   switch (mode) {
     case GST_PAD_MODE_PUSH:
-      aiff->streaming = TRUE;
-      aiff->adapter = gst_adapter_new ();
+      if (active) {
+        aiff->streaming = TRUE;
+        aiff->adapter = gst_adapter_new ();
+      }
       res = TRUE;
       break;
     case GST_PAD_MODE_PULL:
@@ -1651,6 +1678,130 @@ gst_aiff_parse_sink_activate_mode (GstPad * sinkpad, GstObject * parent,
   return res;
 };
 
+static GstFlowReturn
+gst_aiff_parse_flush_data (GstAiffParse * aiff)
+{
+  GstFlowReturn ret = GST_FLOW_OK;
+  guint av;
+
+  if ((av = gst_adapter_available (aiff->adapter)) > 0) {
+    aiff->dataleft = av;
+    aiff->end_offset = aiff->offset + av;
+    ret = gst_aiff_parse_stream_data (aiff);
+  }
+
+  return ret;
+}
+
+
+static gboolean
+gst_aiff_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
+{
+  GstAiffParse *aiff = GST_AIFF_PARSE (parent);
+  gboolean ret = TRUE;
+
+  GST_DEBUG_OBJECT (aiff, "handling %s event", GST_EVENT_TYPE_NAME (event));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CAPS:
+    {
+      /* discard, we'll come up with proper src caps */
+      gst_event_unref (event);
+      break;
+    }
+    case GST_EVENT_SEGMENT:
+    {
+      gint64 start, stop, offset = 0, end_offset = -1;
+      GstSegment segment;
+
+      /* some debug output */
+      gst_event_copy_segment (event, &segment);
+      GST_DEBUG_OBJECT (aiff, "received segment %" GST_SEGMENT_FORMAT,
+          &segment);
+
+      /* now we are either committed to TIME or BYTE format,
+       * and we only expect a BYTE segment, e.g. following a seek */
+      if (segment.format == GST_FORMAT_BYTES) {
+        /* handle (un)signed issues */
+        start = segment.start;
+        stop = segment.stop;
+        if (start > 0) {
+          offset = start;
+          start -= aiff->datastart;
+          start = MAX (start, 0);
+        }
+        if (stop > 0) {
+          end_offset = stop;
+          segment.stop -= aiff->datastart;
+          segment.stop = MAX (stop, 0);
+        }
+        if (aiff->state == AIFF_PARSE_DATA &&
+            aiff->segment.format == GST_FORMAT_TIME) {
+          guint64 bps = aiff->bps;
+
+          /* operating in format TIME, so we can convert */
+          if (bps) {
+            if (start >= 0)
+              start =
+                  gst_util_uint64_scale_ceil (start, GST_SECOND,
+                  (guint64) aiff->bps);
+            if (stop >= 0)
+              stop =
+                  gst_util_uint64_scale_ceil (stop, GST_SECOND,
+                  (guint64) aiff->bps);
+          } else {
+            GST_DEBUG_OBJECT (aiff, "unable to compute segment start/stop");
+            goto exit;
+          }
+        }
+      } else {
+        GST_DEBUG_OBJECT (aiff, "unsupported segment format, ignoring");
+        goto exit;
+      }
+
+      segment.start = start;
+      segment.stop = stop;
+
+      /* accept upstream's notion of segment and distribute along */
+      if (aiff->state == AIFF_PARSE_DATA) {
+        segment.format = aiff->segment.format;
+        segment.time = segment.position = segment.start;
+        segment.duration = aiff->segment.duration;
+        segment.base = gst_segment_to_running_time (&aiff->segment,
+            GST_FORMAT_TIME, aiff->segment.position);
+      }
+
+      gst_segment_copy_into (&segment, &aiff->segment);
+
+      if (aiff->start_segment)
+        gst_event_unref (aiff->start_segment);
+
+      aiff->start_segment = gst_event_new_segment (&segment);
+
+      /* stream leftover data in current segment */
+      if (aiff->state == AIFF_PARSE_DATA)
+        gst_aiff_parse_flush_data (aiff);
+      /* and set up streaming thread for next one */
+      aiff->offset = offset;
+      aiff->end_offset = end_offset;
+      if (aiff->end_offset > 0) {
+        aiff->dataleft = aiff->end_offset - aiff->offset;
+      } else {
+        /* infinity; upstream will EOS when done */
+        aiff->dataleft = G_MAXUINT64;
+      }
+    exit:
+      gst_event_unref (event);
+      break;
+    }
+    default:
+      ret = gst_pad_event_default (aiff->sinkpad, parent, event);
+      break;
+  }
+
+  return ret;
+}
+
 static GstStateChangeReturn
 gst_aiff_parse_change_state (GstElement * element, GstStateChange transition)
 {
