@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 
@@ -48,10 +48,16 @@ typedef struct _MpegTSParse2Class MpegTSParse2Class;
 struct _MpegTSParse2 {
   MpegTSBase parent;
 
+  gboolean have_group_id;
+  guint group_id;
+
   /* Always present source pad */
   GstPad *srcpad;
 
   GList *srcpads;
+
+  /* state */
+  gboolean first;
 };
 
 struct _MpegTSParse2Class {
