@@ -453,10 +453,6 @@ _nal_name (GstH264NalUnitType nal_type)
 }
 #endif
 
-/* SPS/PPS/IDR considered key, all others DELTA;
- * so downstream waiting for keyframe can pick up at SPS/PPS/IDR */
-#define NAL_TYPE_IS_KEY(nt) (((nt) == 5) || ((nt) == 7) || ((nt) == 8))
-
 /* caller guarantees 2 bytes of nal payload */
 static void
 gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
@@ -476,7 +472,6 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
 
   /* we have a peek as well */
   nal_type = nalu->type;
-  h264parse->keyframe |= NAL_TYPE_IS_KEY (nal_type);
 
   GST_DEBUG_OBJECT (h264parse, "processing nal of type %u %s, size %u",
       nal_type, _nal_name (nal_type), nalu->size);
@@ -509,8 +504,10 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
         GST_WARNING_OBJECT (h264parse, "failed to parse PPS:");
 
       /* parameters might have changed, force caps check */
-      GST_DEBUG_OBJECT (h264parse, "triggering src caps check");
-      h264parse->update_caps = TRUE;
+      if (!h264parse->have_pps) {
+        GST_DEBUG_OBJECT (h264parse, "triggering src caps check");
+        h264parse->update_caps = TRUE;
+      }
       h264parse->have_pps = TRUE;
       if (h264parse->push_codec && h264parse->have_sps) {
         /* SPS and PPS found in stream before the first pre_push_frame, no need
@@ -570,18 +567,27 @@ gst_h264_parse_process_nal (GstH264Parse * h264parse, GstH264NalUnit * nalu)
         h264parse->frame_start = TRUE;
       }
       GST_DEBUG_OBJECT (h264parse, "frame start: %i", h264parse->frame_start);
-#ifndef GST_DISABLE_GST_DEBUG
       {
         GstH264SliceHdr slice;
-        GstH264ParserResult pres;
 
         pres = gst_h264_parser_parse_slice_hdr (nalparser, nalu, &slice,
             FALSE, FALSE);
         GST_DEBUG_OBJECT (h264parse,
             "parse result %d, first MB: %u, slice type: %u",
             pres, slice.first_mb_in_slice, slice.type);
+        if (pres == GST_H264_PARSER_OK) {
+          switch (slice.type) {
+            case 2:
+            case 4:
+            case 7:
+            case 9:
+              h264parse->keyframe |= TRUE;
+
+            default:
+              break;
+          }
+        }
       }
-#endif
       if (G_LIKELY (nal_type != GST_H264_NAL_SLICE_IDR &&
               !h264parse->push_codec))
         break;
@@ -1144,7 +1150,7 @@ static void
 gst_h264_parse_update_src_caps (GstH264Parse * h264parse, GstCaps * caps)
 {
   GstH264SPS *sps;
-  GstCaps *sink_caps;
+  GstCaps *sink_caps, *src_caps;
   gboolean modified = FALSE;
   GstBuffer *buf = NULL;
   GstStructure *s = NULL;
@@ -1317,7 +1323,13 @@ gst_h264_parse_update_src_caps (GstH264Parse * h264parse, GstCaps * caps)
       gst_structure_remove_field (s, "codec_data");
       gst_buffer_replace (&h264parse->codec_data, NULL);
     }
-    gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (h264parse), caps);
+
+    src_caps = gst_pad_get_current_caps (GST_BASE_PARSE_SRC_PAD (h264parse));
+    if (!(src_caps && gst_caps_is_strictly_equal (src_caps, caps)))
+      gst_pad_set_caps (GST_BASE_PARSE_SRC_PAD (h264parse), caps);
+
+    if (src_caps)
+      gst_caps_unref (src_caps);
     gst_caps_unref (caps);
   }
 
@@ -1896,10 +1908,13 @@ gst_h264_parse_set_caps (GstBaseParse * parse, GstCaps * caps)
   }
 
   if (format == h264parse->format && align == h264parse->align) {
-    gst_base_parse_set_passthrough (parse, TRUE);
+    /* do not set CAPS and passthrough mode if SPS/PPS have not been parsed */
+    if (h264parse->have_sps && h264parse->have_pps) {
+      gst_base_parse_set_passthrough (parse, TRUE);
 
-    /* we did parse codec-data and might supplement src caps */
-    gst_h264_parse_update_src_caps (h264parse, caps);
+      /* we did parse codec-data and might supplement src caps */
+      gst_h264_parse_update_src_caps (h264parse, caps);
+    }
   } else if (format == GST_H264_PARSE_FORMAT_AVC) {
     /* if input != output, and input is avc, must split before anything else */
     /* arrange to insert codec-data in-stream if needed.
