@@ -60,6 +60,8 @@ static void gst_decklink_src_finalize (GObject * object);
 
 static GstStateChangeReturn
 gst_decklink_src_change_state (GstElement * element, GstStateChange transition);
+static gboolean gst_decklink_src_send_event (GstElement * element,
+    GstEvent * event);
 
 static gboolean gst_decklink_src_audio_src_query (GstPad * pad,
     GstObject * parent, GstQuery * query);
@@ -79,7 +81,7 @@ enum
   PROP_MODE,
   PROP_CONNECTION,
   PROP_AUDIO_INPUT,
-  PROP_DEVICE
+  PROP_DEVICE_NUMBER
 };
 
 static GstStaticPadTemplate gst_decklink_src_audio_src_template =
@@ -104,6 +106,8 @@ gst_decklink_src_class_init (GstDecklinkSrcClass * klass)
   gobject_class->set_property = gst_decklink_src_set_property;
   gobject_class->get_property = gst_decklink_src_get_property;
   gobject_class->finalize = gst_decklink_src_finalize;
+
+  element_class->send_event = GST_DEBUG_FUNCPTR (gst_decklink_src_send_event);
   element_class->change_state =
       GST_DEBUG_FUNCPTR (gst_decklink_src_change_state);
 
@@ -128,14 +132,11 @@ gst_decklink_src_class_init (GstDecklinkSrcClass * klass)
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
               G_PARAM_CONSTRUCT)));
 
-  /* FIXME: should be device-number or so, or turned into a string */
-#if 0
-  g_object_class_install_property (gobject_class, PROP_DEVICE,
-      g_param_spec_int ("device", "Device", "Capture device instance to use",
-          0, G_MAXINT, 0,
+  g_object_class_install_property (gobject_class, PROP_DEVICE_NUMBER,
+      g_param_spec_int ("device-number", "Device number",
+          "Capture device instance to use", 0, G_MAXINT, 0,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS |
               G_PARAM_CONSTRUCT)));
-#endif
 
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_decklink_src_audio_src_template));
@@ -184,7 +185,7 @@ gst_decklink_src_init (GstDecklinkSrc * decklinksrc)
   decklinksrc->mode = GST_DECKLINK_MODE_NTSC;
   decklinksrc->connection = GST_DECKLINK_CONNECTION_SDI;
   decklinksrc->audio_connection = GST_DECKLINK_AUDIO_CONNECTION_AUTO;
-  decklinksrc->device = 0;
+  decklinksrc->device_number = 0;
 
   decklinksrc->stop = FALSE;
   decklinksrc->dropped_frames = 0;
@@ -233,8 +234,8 @@ gst_decklink_src_set_property (GObject * object, guint property_id,
       decklinksrc->audio_connection =
           (GstDecklinkAudioConnectionEnum) g_value_get_enum (value);
       break;
-    case PROP_DEVICE:
-      decklinksrc->device = g_value_get_int (value);
+    case PROP_DEVICE_NUMBER:
+      decklinksrc->device_number = g_value_get_int (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -260,8 +261,8 @@ gst_decklink_src_get_property (GObject * object, guint property_id,
     case PROP_AUDIO_INPUT:
       g_value_set_enum (value, decklinksrc->audio_connection);
       break;
-    case PROP_DEVICE:
-      g_value_set_int (value, decklinksrc->device);
+    case PROP_DEVICE_NUMBER:
+      g_value_set_int (value, decklinksrc->device_number);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -345,6 +346,60 @@ gst_decklink_src_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/* events sent to this element directly, mainly from the application */
+static gboolean
+gst_decklink_src_send_event (GstElement * element, GstEvent * event)
+{
+  GstDecklinkSrc *src;
+  gboolean result = FALSE;
+
+  src = GST_DECKLINK_SRC (element);
+
+  GST_DEBUG_OBJECT (src, "handling event %p %" GST_PTR_FORMAT, event, event);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      g_atomic_int_set (&src->pending_eos, TRUE);
+      GST_INFO_OBJECT (src, "EOS pending");
+      result = TRUE;
+      break;
+      break;
+    case GST_EVENT_TAG:
+    case GST_EVENT_CUSTOM_DOWNSTREAM:
+    case GST_EVENT_CUSTOM_BOTH:
+      /* Insert TAG, CUSTOM_DOWNSTREAM, CUSTOM_BOTH in the dataflow */
+      GST_OBJECT_LOCK (src);
+      src->pending_events = g_list_append (src->pending_events, event);
+      g_atomic_int_set (&src->have_events, TRUE);
+      GST_OBJECT_UNLOCK (src);
+      event = NULL;
+      result = TRUE;
+      break;
+    case GST_EVENT_CUSTOM_DOWNSTREAM_OOB:
+    case GST_EVENT_CUSTOM_BOTH_OOB:
+      /* insert a random custom event into the pipeline */
+      GST_DEBUG_OBJECT (src, "pushing custom OOB event downstream");
+      result = gst_pad_push_event (src->videosrcpad, gst_event_ref (event));
+      result |= gst_pad_push_event (src->audiosrcpad, event);
+      /* we gave away the ref to the event in the push */
+      event = NULL;
+      break;
+    case GST_EVENT_CUSTOM_UPSTREAM:
+      /* drop */
+    case GST_EVENT_SEGMENT:
+      /* sending random SEGMENT downstream can break sync - drop */
+    default:
+      GST_LOG_OBJECT (src, "dropping %s event", GST_EVENT_TYPE_NAME (event));
+      break;
+  }
+
+  /* if we still have a ref to the event, unref it now */
+  if (event)
+    gst_event_unref (event);
+
+  return result;
+}
+
 /* FIXME: post error messages for the misc. failures */
 static gboolean
 gst_decklink_src_start (GstElement * element)
@@ -361,12 +416,12 @@ gst_decklink_src_start (GstElement * element)
 
   GST_DEBUG_OBJECT (decklinksrc, "start");
 
-  decklinksrc->decklink = gst_decklink_get_nth_device (decklinksrc->device);
+  decklinksrc->decklink = gst_decklink_get_nth_device (decklinksrc->device_number);
   if (decklinksrc->decklink == NULL) {
     return FALSE;
   }
 
-  decklinksrc->input = gst_decklink_get_nth_input (decklinksrc->device);
+  decklinksrc->input = gst_decklink_get_nth_input (decklinksrc->device_number);
 
   delegate = new DeckLinkCaptureDelegate ();
   delegate->priv = decklinksrc;
@@ -376,7 +431,7 @@ gst_decklink_src_start (GstElement * element)
     return FALSE;
   }
 
-  decklinksrc->config = gst_decklink_get_nth_config (decklinksrc->device);
+  decklinksrc->config = gst_decklink_get_nth_config (decklinksrc->device_number);
   config = decklinksrc->config;
 
   switch (decklinksrc->connection) {
@@ -491,6 +546,12 @@ gst_decklink_src_stop (GstElement * element)
   decklinksrc->input->DisableVideoInput ();
   decklinksrc->input->DisableAudioInput ();
 
+  g_list_free_full (decklinksrc->pending_events,
+      (GDestroyNotify) gst_mini_object_unref);
+  decklinksrc->pending_events = NULL;
+  decklinksrc->have_events = FALSE;
+  decklinksrc->pending_eos = FALSE;
+
   return TRUE;
 }
 
@@ -627,6 +688,7 @@ gst_decklink_src_send_initial_events (GstDecklinkSrc * src)
 {
   GstSegment segment;
   GstEvent *event;
+  guint group_id;
   guint32 audio_id, video_id;
   gchar stream_id[9];
 
@@ -636,11 +698,16 @@ gst_decklink_src_send_initial_events (GstDecklinkSrc * src)
   while (video_id == audio_id)
     video_id = g_random_int ();
 
+  group_id = gst_util_group_id_next ();
   g_snprintf (stream_id, sizeof (stream_id), "%08x", audio_id);
-  gst_pad_push_event (src->audiosrcpad, gst_event_new_stream_start (stream_id));
+  event = gst_event_new_stream_start (stream_id);
+  gst_event_set_group_id (event, group_id);
+  gst_pad_push_event (src->audiosrcpad, event);
 
   g_snprintf (stream_id, sizeof (stream_id), "%08x", video_id);
-  gst_pad_push_event (src->videosrcpad, gst_event_new_stream_start (stream_id));
+  event = gst_event_new_stream_start (stream_id);
+  gst_event_set_group_id (event, group_id);
+  gst_pad_push_event (src->videosrcpad, event);
 
   /* segment */
   gst_segment_init (&segment, GST_FORMAT_TIME);
@@ -670,7 +737,7 @@ gst_decklink_src_task (void *priv)
   void *data;
   gsize data_size;
   int n_samples;
-  GstFlowReturn ret;
+  GstFlowReturn video_flow, audio_flow, flow;
   const GstDecklinkMode *mode;
   gboolean discont = FALSE;
 
@@ -711,6 +778,25 @@ gst_decklink_src_task (void *priv)
     decklinksrc->started = TRUE;
   }
 
+  if (g_atomic_int_get (&decklinksrc->have_events)) {
+    GList *l;
+
+    GST_OBJECT_LOCK (decklinksrc);
+    for (l = decklinksrc->pending_events; l != NULL; l = l->next) {
+      GstEvent *event = GST_EVENT (l->data);
+
+      GST_DEBUG_OBJECT (decklinksrc, "pushing %s event",
+          GST_EVENT_TYPE_NAME (event));
+      gst_pad_push_event (decklinksrc->videosrcpad, gst_event_ref (event));
+      gst_pad_push_event (decklinksrc->audiosrcpad, event);
+      l->data = NULL;
+    }
+    g_list_free (decklinksrc->pending_events);
+    decklinksrc->pending_events = NULL;
+    g_atomic_int_set (&decklinksrc->have_events, FALSE);
+    GST_OBJECT_UNLOCK (decklinksrc);
+  }
+
   mode = gst_decklink_get_mode (decklinksrc->mode);
 
   video_frame->GetBytes (&data);
@@ -755,15 +841,7 @@ gst_decklink_src_task (void *priv)
   else
     GST_BUFFER_FLAG_UNSET (buffer, GST_BUFFER_FLAG_DISCONT);
 
-  /* FIXME: proper flow aggregation with audio flow */
-  ret = gst_pad_push (decklinksrc->videosrcpad, buffer);
-  if (!(ret == GST_FLOW_OK || ret == GST_FLOW_NOT_LINKED ||
-          ret == GST_FLOW_FLUSHING)) {
-    GST_ELEMENT_ERROR (decklinksrc, STREAM, FAILED,
-        ("Internal data stream error."),
-        ("stream stopped, reason %s", gst_flow_get_name (ret)));
-      goto pause;
-  }
+  video_flow = gst_pad_push (decklinksrc->videosrcpad, buffer);
 
   if (gst_pad_is_linked (decklinksrc->audiosrcpad)) {
     n_samples = audio_frame->GetSampleFrameCount ();
@@ -783,17 +861,40 @@ gst_decklink_src_task (void *priv)
 
     decklinksrc->num_audio_samples += n_samples;
 
-  /* FIXME: proper flow aggregation with video flow */
-    ret = gst_pad_push (decklinksrc->audiosrcpad, audio_buffer);
-    if (!(ret == GST_FLOW_OK || ret == GST_FLOW_NOT_LINKED ||
-            ret == GST_FLOW_FLUSHING)) {
-      GST_ELEMENT_ERROR (decklinksrc, STREAM, FAILED,
-          ("Internal data stream error."),
-          ("stream stopped, reason %s", gst_flow_get_name (ret)));
-      goto pause;
-    }
+    if (discont)
+      GST_BUFFER_FLAG_SET (audio_buffer, GST_BUFFER_FLAG_DISCONT);
+    else
+      GST_BUFFER_FLAG_UNSET (audio_buffer, GST_BUFFER_FLAG_DISCONT);
+
+    audio_flow = gst_pad_push (decklinksrc->audiosrcpad, audio_buffer);
+  } else {
+    audio_flow = GST_FLOW_NOT_LINKED;
   }
 
+  if (audio_flow == GST_FLOW_NOT_LINKED)
+    flow = video_flow;
+  else if (video_flow == GST_FLOW_NOT_LINKED)
+    flow = audio_flow;
+  else if (video_flow == GST_FLOW_FLUSHING || audio_flow == GST_FLOW_FLUSHING)
+    flow = GST_FLOW_FLUSHING;
+  else if (video_flow < GST_FLOW_EOS)
+    flow = video_flow;
+  else if (audio_flow < GST_FLOW_EOS)
+    flow = audio_flow;
+  else if (video_flow == GST_FLOW_EOS || audio_flow == GST_FLOW_EOS)
+    flow = GST_FLOW_EOS;
+  else
+    flow = video_flow;
+
+  if (g_atomic_int_compare_and_exchange (&decklinksrc->pending_eos, TRUE,
+      FALSE)) {
+    GST_INFO_OBJECT (decklinksrc, "EOS pending");
+    flow = GST_FLOW_EOS;
+  }
+
+  if (flow != GST_FLOW_OK)
+    goto pause;
+
 done:
 
   if (audio_frame)
@@ -803,15 +904,15 @@ done:
 
 pause:
   {
-    const gchar *reason = gst_flow_get_name (ret);
+    const gchar *reason = gst_flow_get_name (flow);
     GstEvent *event = NULL;
 
     GST_DEBUG_OBJECT (decklinksrc, "pausing task, reason %s", reason);
     gst_task_pause (decklinksrc->task);
-    if (ret == GST_FLOW_EOS) {
+    if (flow == GST_FLOW_EOS) {
       /* perform EOS logic (very crude, we don't even keep a GstSegment) */
       event = gst_event_new_eos ();
-    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
+    } else if (flow == GST_FLOW_NOT_LINKED || flow < GST_FLOW_EOS) {
       event = gst_event_new_eos ();
       /* for fatal errors we post an error message, post the error
        * first so the app knows about the error first.
@@ -821,7 +922,7 @@ pause:
        * a flushing seek. */
       GST_ELEMENT_ERROR (decklinksrc, STREAM, FAILED,
           ("Internal data flow error."),
-          ("streaming task paused, reason %s (%d)", reason, ret));
+          ("streaming task paused, reason %s (%d)", reason, flow));
     }
     if (event != NULL) {
       GST_INFO_OBJECT (decklinksrc->videosrcpad, "pushing EOS event");
